@@ -23,7 +23,6 @@ import operator
 import re
 
 import msgpack
-import numpy
 import pandas
 import six
 
@@ -51,6 +50,14 @@ class UnAggregableTimeseries(Exception):
         super(UnAggregableTimeseries, self).__init__(reason)
 
 
+class UnknownAggregationMethod(Exception):
+    """Error raised when the aggregation method is unknown."""
+    def __init__(self, agg):
+        self.aggregation_method = agg
+        super(UnknownAggregationMethod, self).__init__(
+            "Unknown aggregation method `%s'" % agg)
+
+
 class SerializableMixin(object):
 
     @classmethod
@@ -211,7 +218,7 @@ class BoundTimeSerie(TimeSerie):
         return basic
 
     def _first_block_timestamp(self):
-        ts = self.ts.resample(self.block_size)
+        ts = self.ts[-1:].resample(self.block_size)
         return (ts.index[-1] - (self.block_size * self.back_window))
 
     def _truncate(self):
@@ -227,13 +234,6 @@ class AggregatedTimeSerie(TimeSerie):
 
     _AGG_METHOD_PCT_RE = re.compile(r"([1-9][0-9]?)pct")
 
-    @staticmethod
-    def _percentile(a, q):
-        # TODO(jd) Find a way to compute all the percentile in one pass as
-        # numpy can do numpy.percentile(a, q=[75, 90, 95])
-        if len(a) > 0:
-            return numpy.percentile(a, q)
-
     def __init__(self, timestamps=None, values=None,
                  max_size=None,
                  sampling=None, aggregation_method='mean'):
@@ -245,18 +245,20 @@ class AggregatedTimeSerie(TimeSerie):
         """
         super(AggregatedTimeSerie, self).__init__(timestamps, values)
 
-        self.aggregation_method = aggregation_method
-
         m = self._AGG_METHOD_PCT_RE.match(aggregation_method)
 
         if m:
-            self.aggregation_method_func = functools.partial(
-                self._percentile, q=float(m.group(1)))
+            self.q = float(m.group(1)) / 100
+            self.aggregation_method_func_name = 'quantile'
         else:
-            self.aggregation_method_func = aggregation_method
+            if not hasattr(pandas.core.groupby.SeriesGroupBy,
+                           aggregation_method):
+                raise UnknownAggregationMethod(aggregation_method)
+            self.aggregation_method_func_name = aggregation_method
 
         self.sampling = pandas.tseries.frequencies.to_offset(sampling)
         self.max_size = max_size
+        self.aggregation_method = aggregation_method
 
     def __eq__(self, other):
         return (isinstance(other, AggregatedTimeSerie)
@@ -295,14 +297,30 @@ class AggregatedTimeSerie(TimeSerie):
             # Remove empty points if any that could be added by aggregation
             self.ts = self.ts.dropna()[-self.max_size:]
 
+    @staticmethod
+    def _round_timestamp(ts, freq):
+        return pandas.Timestamp(
+            (ts.value // freq.delta.value) * freq.delta.value)
+
     def _resample(self, after):
         if self.sampling:
-            self.ts = self.ts[after:].resample(
-                self.sampling,
-                how=self.aggregation_method_func).combine_first(
-                    self.ts[:after][:-1])
+            # Group by the sampling, and then apply the aggregation method on
+            # the points after `after'
+            groupedby = self.ts[after:].groupby(
+                functools.partial(self._round_timestamp,
+                                  freq=self.sampling))
+            agg_func = getattr(groupedby, self.aggregation_method_func_name)
+            if self.aggregation_method_func_name == 'quantile':
+                aggregated = agg_func(self.q)
+            else:
+                aggregated = agg_func()
+            # Now combine the result with the rest of the point – everything
+            # that is before `after'
+            self.ts = aggregated.combine_first(self.ts[:after][:-1])
 
     def update(self, ts):
+        if ts.ts.empty:
+            return
         index = ts.ts.index
         first_timestamp = index[0]
         last_timestamp = index[-1]
@@ -482,8 +500,8 @@ class TimeSerieArchive(SerializableMixin):
             agg_timeserie = agg_timeserie[
                 agg_timeserie['timestamp'] <= right_boundary_ts]
 
-        points = (agg_timeserie.sort(['granularity', 'timestamp'],
-                                     ascending=[0, 1]).itertuples())
+        points = (agg_timeserie.sort_values(by=['granularity', 'timestamp'],
+                                            ascending=[0, 1]).itertuples())
         return [(timestamp, granularity, value)
                 for __, timestamp, granularity, value in points]
 
