@@ -19,6 +19,7 @@ import operator
 import os.path
 import uuid
 
+import oslo_db.api
 from oslo_db import exception
 from oslo_db.sqlalchemy import models
 from oslo_db.sqlalchemy import session
@@ -134,12 +135,14 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                 raise indexer.ArchivePolicyInUse(name)
             raise
 
-    def get_metrics(self, uuids):
+    def get_metrics(self, uuids, active_only=True):
         if not uuids:
             return []
         session = self.engine_facade.get_session()
         query = session.query(Metric).filter(Metric.id.in_(uuids)).options(
             sqlalchemy.orm.joinedload('resource'))
+        if active_only:
+            query = query.filter(Metric.status == 'active')
 
         metrics = list(query.all())
         session.expunge_all()
@@ -163,7 +166,8 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
     def list_archive_policy_rules(self):
         session = self.engine_facade.get_session()
-        aps = session.query(ArchivePolicyRule).all()
+        aps = session.query(ArchivePolicyRule).order_by(
+            ArchivePolicyRule.metric_pattern.desc()).all()
         session.expunge_all()
         return aps
 
@@ -219,7 +223,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
     def list_metrics(self, user_id=None, project_id=None, details=False,
                      **kwargs):
         session = self.engine_facade.get_session()
-        q = session.query(Metric)
+        q = session.query(Metric).filter(Metric.status == 'active')
         if user_id is not None:
             q = q.filter(Metric.created_by_user_id == user_id)
         if project_id is not None:
@@ -273,6 +277,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         session.expunge_all()
         return r
 
+    @oslo_db.api.retry_on_deadlock
     def update_resource(self, resource_type,
                         resource_id, ended_at=_marker, metrics=_marker,
                         append_metrics=False,
@@ -328,7 +333,8 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                 if metrics is not _marker:
                     if not append_metrics:
                         session.query(Metric).filter(
-                            Metric.resource_id == resource_id).update(
+                            Metric.resource_id == resource_id,
+                            Metric.status == 'active').update(
                                 {"resource_id": None})
                     self._set_metrics_for_resource(session, r, metrics)
         except exception.DBConstraintError as e:
@@ -350,6 +356,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                 try:
                     update = session.query(Metric).filter(
                         Metric.id == value,
+                        Metric.status == 'active',
                         (Metric.created_by_user_id
                          == r.created_by_user_id),
                         (Metric.created_by_project_id
@@ -380,18 +387,19 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
         session.expire(r, ['metrics'])
 
-    def delete_resource(self, resource_id, delete_metrics=None):
+    def delete_resource(self, resource_id):
         session = self.engine_facade.get_session()
         with session.begin():
-            q = session.query(Resource).filter(
-                Resource.id == resource_id).options(
-                    sqlalchemy.orm.joinedload('metrics'))
-            r = q.first()
-            if r is None:
+            # We are going to delete the resource; the on delete will set the
+            # resource_id of the attached metrics to NULL, we just have to mark
+            # their status as 'delete'
+            session.query(Metric).filter(
+                Metric.resource_id == resource_id).update(
+                    {"status": "delete"})
+            if session.query(Resource).filter(
+                    Resource.id == resource_id).options(
+                        sqlalchemy.orm.joinedload('metrics')).delete() == 0:
                 raise indexer.NoSuchResource(resource_id)
-            if delete_metrics is not None:
-                delete_metrics(self.get_metrics([m.id for m in r.metrics]))
-            q.delete()
 
     def get_resource(self, resource_type, resource_id, with_metrics=False):
         resource_cls = self._resource_type_to_class(resource_type)
@@ -436,7 +444,9 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             properties={
                 'metrics': sqlalchemy.orm.relationship(
                     Metric,
-                    primaryjoin=Metric.resource_id == stmt.c.id,
+                    primaryjoin=sqlalchemy.and_(
+                        Metric.resource_id == stmt.c.id,
+                        Metric.status == 'active'),
                     foreign_keys=Metric.resource_id)
             })
 
@@ -532,9 +542,15 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         session.expunge_all()
         return all_resources
 
+    def expunge_metric(self, id):
+        session = self.engine_facade.get_session()
+        if session.query(Metric).filter(Metric.id == id).delete == 0:
+            raise indexer.NoSuchMetric(id)
+
     def delete_metric(self, id):
         session = self.engine_facade.get_session()
-        if session.query(Metric).filter(Metric.id == id).delete() == 0:
+        if session.query(Metric).filter(
+                Metric.id == id).update({"status": "delete"}) == 0:
             raise indexer.NoSuchMetric(id)
 
 
