@@ -13,12 +13,14 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import logging
 import operator
 from oslo_config import cfg
 from oslo_utils import timeutils
 from stevedore import driver
 
 from gnocchi import exceptions
+from gnocchi import indexer
 
 
 OPTS = [
@@ -29,8 +31,14 @@ OPTS = [
                default=5,
                help="How many seconds to wait between "
                "new metric measure processing"),
+    cfg.IntOpt('metric_reporting_delay',
+               default=10,
+               help="How many seconds to wait between "
+               "metric ingestion reporting"),
 ]
 
+LOG = logging.getLogger(__name__)
+
 
 class Measure(object):
     def __init__(self, timestamp, value):
@@ -141,6 +149,35 @@ class StorageDriver(object):
     def stop():
         pass
 
+    def process_background_tasks(self, index):
+        LOG.debug("Processing new and to delete measures")
+        try:
+            self.process_measures(index)
+        except Exception:
+            LOG.error("Unexpected error during measures processing",
+                      exc_info=True)
+        LOG.debug("Expunging deleted metrics")
+        try:
+            self.expunge_metrics(index)
+        except Exception:
+            LOG.error("Unexpected error during deleting metrics",
+                      exc_info=True)
+
+    def expunge_metrics(self, index):
+        metrics_to_expunge = index.list_metrics(status='delete')
+        for m in metrics_to_expunge:
+            try:
+                self.delete_metric(m)
+            except Exception:
+                LOG.error("Unable to expunge metric %s from storage" % m,
+                          exc_info=True)
+            try:
+                self.index.expunge_metric(m.id)
+            except indexer.NoSuchMetric:
+                # It's possible another process deleted the metric in the mean
+                # time, not a big deal
+                pass
+
     @staticmethod
     def add_measures(metric, measures):
         """Add a measure to a metric.
@@ -158,15 +195,26 @@ class StorageDriver(object):
         the measures sent to metrics. This is used for that.
         """
 
+    @staticmethod
+    def measures_report():
+        """Return a report of pending to process measures.
+
+        Only usefull for drivers that process measurements in background
+
+        :return: {metric_id: pending_measures_count}
+        """
+        raise exceptions.NotImplementedError
+
     @staticmethod
     def get_measures(metric, from_timestamp=None, to_timestamp=None,
-                     aggregation='mean'):
+                     aggregation='mean', granularity=None):
         """Get a measure to a metric.
 
         :param metric: The metric measured.
         :param from timestamp: The timestamp to get the measure from.
         :param to timestamp: The timestamp to get the measure to.
         :param aggregation: The type of aggregation to retrieve.
+        :param granularity: The granularity to retrieve.
         """
         if aggregation not in metric.archive_policy.aggregation_methods:
             raise AggregationDoesNotExist(metric, aggregation)
