@@ -105,37 +105,14 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
              datetime.datetime(2014, 1, 1, 12, 0, 9)],
             [3, 5, 6])
 
-    def test_0_percentile(self):
-        ts = carbonara.AggregatedTimeSerie(sampling='1Min',
-                                           aggregation_method='0pct')
-        self.assertRaises(AttributeError,
-                          ts.update,
-                          carbonara.TimeSerie.from_tuples(
-                              [(datetime.datetime(2014, 1, 1, 12, 0, 0), 3),
-                               (datetime.datetime(2014, 1, 1, 12, 0, 4), 5),
-                               (datetime.datetime(2014, 1, 1, 12, 0, 9), 6)]))
+    def test_bad_percentile(self):
+        for bad_percentile in ('0pct', '100pct', '-1pct', '123pct'):
+            self.assertRaises(carbonara.UnknownAggregationMethod,
+                              carbonara.AggregatedTimeSerie,
+                              sampling='1Min',
+                              aggregation_method=bad_percentile)
 
-    def test_100_percentile(self):
-        ts = carbonara.AggregatedTimeSerie(sampling='1Min',
-                                           aggregation_method='100pct')
-        self.assertRaises(AttributeError,
-                          ts.update,
-                          carbonara.TimeSerie.from_tuples(
-                              [(datetime.datetime(2014, 1, 1, 12, 0, 0), 3),
-                               (datetime.datetime(2014, 1, 1, 12, 0, 4), 5),
-                               (datetime.datetime(2014, 1, 1, 12, 0, 9), 6)]))
-
-    def test_123_percentile(self):
-        ts = carbonara.AggregatedTimeSerie(sampling='1Min',
-                                           aggregation_method='123pct')
-        self.assertRaises(AttributeError,
-                          ts.update,
-                          carbonara.TimeSerie.from_tuples(
-                              [(datetime.datetime(2014, 1, 1, 12, 0, 0), 3),
-                               (datetime.datetime(2014, 1, 1, 12, 0, 4), 5),
-                               (datetime.datetime(2014, 1, 1, 12, 0, 9), 6)]))
-
-    def test_74_percentile(self):
+    def test_74_percentile_serialized(self):
         ts = carbonara.AggregatedTimeSerie(sampling='1Min',
                                            aggregation_method='74pct')
         ts.update(carbonara.TimeSerie.from_tuples(
@@ -146,6 +123,17 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
         self.assertEqual(1, len(ts))
         self.assertEqual(5.48, ts[datetime.datetime(2014, 1, 1, 12, 0, 0)])
 
+        # Serialize and unserialize
+        ts = carbonara.AggregatedTimeSerie.unserialize(ts.serialize())
+
+        ts.update(carbonara.TimeSerie.from_tuples(
+            [(datetime.datetime(2014, 1, 1, 12, 0, 0), 3),
+             (datetime.datetime(2014, 1, 1, 12, 0, 4), 5),
+             (datetime.datetime(2014, 1, 1, 12, 0, 9), 6)]))
+
+        self.assertEqual(1, len(ts))
+        self.assertEqual(5.48, ts[datetime.datetime(2014, 1, 1, 12, 0, 0)])
+
     def test_95_percentile(self):
         ts = carbonara.AggregatedTimeSerie(sampling='1Min',
                                            aggregation_method='95pct')
@@ -234,6 +222,15 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
 
 class TestTimeSerieArchive(base.BaseTestCase):
 
+    def test_empty_update(self):
+        tsc = carbonara.TimeSerieArchive.from_definitions(
+            [(60, 10),
+             (300, 6)])
+        tsb = carbonara.BoundTimeSerie(block_size=tsc.max_block_size)
+        tsb.set_values([], before_truncate_callback=tsc.update)
+
+        self.assertEqual([], tsc.fetch())
+
     def test_fetch(self):
         tsc = carbonara.TimeSerieArchive.from_definitions(
             [(60, 10),
